@@ -27,10 +27,15 @@
 #define PHP_DS_NS(cls) PHP_DS_NS_NAME #cls
 
 /**
- *
+ * This is useful for functions that require the string and its length.
  */
 #define STR_AND_LEN(str) str, sizeof(str) - 1
 
+/**
+ * Slightly faster check when the given zval is probably a boolean.
+ */
+#define EXPECTED_BOOL_IS_TRUE(z) (Z_TYPE_P(z) != IS_FALSE && zend_is_true(z))
+
 /**
  * Combined class, name, and arginfo method entry.
  */
@@ -67,7 +72,7 @@ do {                                        \
 } while (0)
 
 /**
- *
+ * Swaps two zval's.
  */
 #define SWAP_ZVAL(a, b) \
 do {                    \
@@ -77,20 +82,19 @@ do {                    \
 } while (0)
 
 /**
- *
+ * Adds the given zval "val" to "sum".
  */
-#define DS_ADD_TO_SUM(value, sum)                                       \
+#define DS_ADD_TO_SUM(val, sum)                                         \
 do {                                                                    \
-    zval number;                                                        \
-                                                                        \
-    if (Z_TYPE_P(value) == IS_ARRAY || Z_TYPE_P(value) == IS_OBJECT) {  \
-        continue;                                                       \
+    if (Z_TYPE_P(val) == IS_LONG || Z_TYPE_P(val) == IS_DOUBLE) {       \
+        fast_add_function(sum, sum, val);                               \
+    } else {                                                            \
+        zval _num;                                                      \
+        ZVAL_COPY(&_num, val);                                          \
+        convert_scalar_to_number(&_num);                                \
+        fast_add_function(sum, sum, &_num);                             \
     }                                                                   \
-                                                                        \
-    ZVAL_COPY(&number, value);                                          \
-    convert_scalar_to_number(&number);                                  \
-    fast_add_function(sum, sum, &number);                               \
-} while (0)                                                             \
+} while (0)
 
 /**
  * Used to replace a buffer with a new one.
@@ -105,14 +109,14 @@ do {                                \
 /**
  * Copies 'len' values from 'src' into 'dst'.
  */
-#define COPY_ZVAL_BUFFER(dst, src, len) \
-do { \
-    zval *_src = src; \
-    zval *_dst = dst; \
-    zval *_end = src + len; \
-    for (; _src != _end; ++_src, ++_dst) { \
-        ZVAL_COPY(_dst, _src); \
-    } \
+#define COPY_ZVAL_BUFFER(dst, src, len)     \
+do {                                        \
+    zval *_src = src;                       \
+    zval *_dst = dst;                       \
+    zval *_end = src + len;                 \
+    for (; _src != _end; ++_src, ++_dst) {  \
+        ZVAL_COPY(_dst, _src);              \
+    }                                       \
 } while (0)
 
 /**
@@ -133,9 +137,9 @@ do {                                 \
     return;                          \
 } while (0)
 
-#define SERIALIZE_SET_ZSTR(s) \
-*buffer = (unsigned char *) estrndup(ZSTR_VAL((s)), ZSTR_LEN((s))); \
-*length = ZSTR_LEN((s));
+#define SERIALIZE_SET_ZSTR(s)                                           \
+    *buffer = (unsigned char *) estrndup(ZSTR_VAL((s)), ZSTR_LEN((s))); \
+    *length = ZSTR_LEN((s));
 
 #define PHP_DS_SERIALIZE_FUNCIONS(name) \
 int name##_serialize(                   \
@@ -225,6 +229,11 @@ void ds_throw_exception(zend_class_entry *ce, const char *format, ...);
 
 /*****************************************************************************/
 
+/**
+ * Returns the next power of 2 greater than or equal to n.
+ */
+uint32_t ds_next_power_of_2(uint32_t n, uint32_t min);
+
 /**
  * Similar to 'implode', joins a zval buffer using an optional 'glue'.
  * Use NULL and 0 for 'str' and 'len' to indicate an optional glue.
@@ -259,7 +268,7 @@ zval *ds_allocate_zval_buffer(zend_long length);
  * @param  length The resulting length of the buffer.
  * @param  used   Number of slots currently in use in the given buffer.
  */
-zval *ds_reallocate_zval_buffer(zval *buffer, zend_long length, zend_long used);
+zval *ds_reallocate_zval_buffer(zval *buffer, zend_long length, zend_long current, zend_long used);
 
 /**
  * Sorts a zval buffer in place using the default internal compare_func.
@@ -301,4 +310,9 @@ bool ds_is_traversable(zval *value);
  */
 void smart_str_appendz(smart_str *buffer, zval *value);
 
+/**
+ *
+ */
+bool ds_special_is_equal(zval *op1, zval *op2);
+
 #endif
