@@ -1,27 +1,11 @@
 #include "../common.h"
 
-#include "ds_pair.h"
 #include "ds_htable.h"
 #include "ds_set.h"
 #include "ds_vector.h"
 
 #include "../php/classes/php_hashable_ce.h"
 
-static uint32_t next_power_of_2(uint32_t size)
-{
-    uint32_t c = MAX(size, DS_HTABLE_MIN_CAPACITY);
-
-    c--;
-    c |= c >> 1;
-    c |= c >> 2;
-    c |= c >> 4;
-    c |= c >> 8;
-    c |= c >> 16;
-    c++;
-
-    return c;
-}
-
 static inline ds_htable_bucket_t *ds_htable_allocate_buckets(uint32_t capacity)
 {
     return ecalloc(capacity, sizeof(ds_htable_bucket_t));
@@ -252,11 +236,7 @@ static uint32_t get_array_hash(zval *array)
 
 static inline uint32_t get_spl_object_hash(zval *obj)
 {
-    zend_string *str = php_spl_object_hash(obj);
-    uint32_t hash = get_string_hash(str);
-    zend_string_free(str);
-
-    return hash;
+    return Z_OBJ_HANDLE_P(obj);
 }
 
 static inline uint32_t get_resource_hash(zval *resource)
@@ -264,6 +244,21 @@ static inline uint32_t get_resource_hash(zval *resource)
     return Z_RES_HANDLE_P(resource);
 }
 
+static inline uint32_t get_double_hash(zval *value)
+{
+    union ulld {
+        unsigned long long ull;
+        double d;
+    } hack;
+
+    hack.d = Z_DVAL_P(value);
+    if (hack.d == -0.0) {
+        hack.d = 0.0;
+    }
+
+    return (uint32_t)(hack.ull ^ (hack.ull >> 32));
+}
+
 static uint32_t get_object_hash(zval *obj)
 {
     if (implements_hashable(obj)) {
@@ -275,7 +270,7 @@ static uint32_t get_object_hash(zval *obj)
                 return Z_LVAL(hash);
 
             case IS_DOUBLE:
-                return zval_get_long(&hash);
+                return get_double_hash(&hash);
 
             case IS_STRING:
                 return get_string_zval_hash(&hash);
@@ -304,7 +299,7 @@ static uint32_t get_hash(zval *value)
             return Z_LVAL_P(value);
 
         case IS_DOUBLE:
-            return zval_get_long(value);
+            return get_double_hash(value);
 
         case IS_STRING:
             return get_string_zval_hash(value);
@@ -617,10 +612,17 @@ static inline void ds_htable_increase_capacity(ds_htable_t *table)
     ds_htable_rehash(table);
 }
 
+static inline zend_long ds_htable_get_capacity_for_size(zend_long size)
+{
+    return ds_next_power_of_2(size, DS_HTABLE_MIN_CAPACITY);
+}
+
 void ds_htable_ensure_capacity(ds_htable_t *table, uint32_t capacity)
 {
+    capacity = ds_htable_get_capacity_for_size(capacity);
+
     if (capacity > table->capacity) {
-        ds_htable_realloc(table, next_power_of_2(capacity));
+        ds_htable_realloc(table, capacity);
         ds_htable_rehash(table);
     }
 }
@@ -1208,7 +1210,7 @@ int ds_htable_unserialize(ds_htable_t *table, const unsigned char *buffer, size_
 
     PHP_VAR_UNSERIALIZE_INIT(unserialize_data);
 
-    while (*pos != '}') {
+    while (pos != end) {
 
         zval *key   = var_tmp_var(&unserialize_data);
         zval *value = var_tmp_var(&unserialize_data);
@@ -1228,10 +1230,6 @@ int ds_htable_unserialize(ds_htable_t *table, const unsigned char *buffer, size_
         ds_htable_put(table, key, value);
     }
 
-    if (pos != end) {
-        goto error;
-    }
-
     PHP_VAR_UNSERIALIZE_DESTROY(unserialize_data);
     return SUCCESS;
 
